#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''AS Configurations v1 action implementations'''
import argparse
import base64
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print_detail(obj):
    info = {}
    info['name'] = obj.name
    info['id'] = obj.id
    info['create_time'] = obj.create_time
    instance_config = obj.instance_config
    if instance_config:
        info['instance_name'] = instance_config.instance_name
        info['instance_id'] = instance_config.instance_id
        info['flavor_id'] = instance_config.flavor_id
        info['image_id'] = instance_config.image_id
        info['key_name'] = instance_config.key_name
        info['public_ip'] = instance_config.public_ip
        info['user_data'] = instance_config.user_data
        info['metadata'] = instance_config.metadata
        disks = []
        for disk in instance_config.disk:
            dsk = {
                'size': disk['size'],
                'volume_type': disk['volume_type'],
                'disk_type': disk['disk_type'],
            }
            disks.append(dsk)
        info['disk'] = disks

    return info


class ListAutoScalingConfig(command.Lister):
    _description = _('List AutoScaling Configurations')
    columns = ('ID', 'Name')

    def get_parser(self, prog_name):
        parser = super(ListAutoScalingConfig, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.marker:
            args['marker'] = parsed_args.marker

        client = self.app.client_manager.auto_scaling

        data = client.configs(**args)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowAutoScalingConfig(command.ShowOne):
    _description = _('Shows details of a AutoScalinig group')
    columns = ['ID', 'Name', 'instance_id', 'instance_name',
               'flavor_id', 'image_id', 'disk',
               'key_name', 'public_ip', 'user_data', 'metadata'
               ]

    def get_parser(self, prog_name):
        parser = super(ShowAutoScalingConfig, self).get_parser(prog_name)
        parser.add_argument(
            'config',
            metavar='<config>',
            help=_('ID or name of the configuration group')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        obj = client.find_config(parsed_args.config, ignore_missing=False)

        # display_columns, columns = _get_columns(obj)
        # data = utils.get_item_properties(
        #     obj, columns, formatters={'instance_config': _format_instance})

        fmt = set_attributes_for_print_detail(obj)
        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(
            fmt, self.columns, formatters={})

        return (self.columns, data)


class CreateAutoScalingConfig(command.ShowOne):
    _description = _('Creates AutoScalinig group')
    columns = ['ID', 'Name', 'instance_id', 'instance_name',
               'flavor_id', 'image_id', 'disk',
               'key_name', 'public_ip', 'user_data', 'metadata'
               ]

    def get_parser(self, prog_name):
        parser = super(CreateAutoScalingConfig, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('AS Configuration name')
        )
        group1 = parser.add_argument_group(
            'ECS', 'New scpecification template')
        group1.add_argument(
            '--flavor',
            metavar='<flavor>',
            help=_('Flavor ID or Name for the ECS instance')
        )
        group1.add_argument(
            '--image-id',
            metavar='<image_id>',
            help=_('Image ID for the ECS instance to be created')
        )
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--instance-id',
            metavar='<instance_id>',
            help=_('AS Configuration name\n'
                   'Is mutually exclusive with ECS group')
        )

        parser.add_argument(
            '--disk',
            metavar='<disk>',
            action='append',
            help=_(
                'Disk information to attach to the instance.\n'
                'format = `DISK_TYPE`,`VOLUME_TYPE`,`SIZE`\n'
                '`DISK_TYPE` can be in [SYS, DATA] and identifies '
                'whether disk should be a system or data disk\n'
                '`VOLUME_TYPE` can be in: \n'
                '* `SATA` = Common I/O \n'
                '* `SAS` = High I/O \n'
                '* `SSD` = Ultra-High I/O \n'
                '`SIZE` is size in Gb\n'
                '(Repeat multiple times for multiple disks)')
        )

        parser.add_argument(
            '--key',
            metavar='<key>',
            help=_('Key name for the new ECS instance')
        )
        parser.add_argument(
            '--public-ip-bandwith',
            metavar='<public_ip_bandwith>',
            type=int,
            help=_('Defines EIP Bandwith (Mbit/s) to be attached '
                   'to the new ECS instance')
        )
        parser.add_argument(
            '--user-data',
            metavar='<user_data>',
            help=_('Path to the cloud-init user_data file')
        )
        parser.add_argument(
            '--metadata',
            metavar='<metadata>',
            help=_('User defined key=value pair '
                   'format KEY=VALUE '
                   'Cannot contain dot (`.`) or start with `$`')
        )
        return parser

    def take_action(self, parsed_args):

        config_attrs = {}
        if parsed_args.instance_id:
            config_attrs['instance_id'] = parsed_args.instance_id
        else:
            if not all(
                    [parsed_args.flavor,
                     parsed_args.image_id,
                     parsed_args.disk]):
                msg = _('Either instance_id or all of the '
                        '[flavor, image, disk] '
                        'should be given')
                raise argparse.ArgumentTypeError(msg)
            # config_attrs['name'] = parsed_args.name
            config_attrs['imageRef'] = parsed_args.image_id
            config_attrs['flavorRef'] = parsed_args.flavor
        config_attrs['key_name'] = parsed_args.key

        config_attrs['disk'] = []
        for disk in parsed_args.disk:
            disk_parts = disk.split(',')
            disk_data = {}
            if 3 == len(disk_parts):
                if disk_parts[0] in ('SYS', 'DATA'):
                    disk_data['disk_type'] = disk_parts[0]
                else:
                    msg = _('Disk Type is not in (SYS, DATA)')
                    raise argparse.ArgumentTypeError(msg)
                if disk_parts[1] in ('SATA', 'SAS', 'SSD'):
                    disk_data['volume_type'] = disk_parts[1]
                else:
                    msg = _('Volume Type is not in (SATA, SAS, SSD)')
                    raise argparse.ArgumentTypeError(msg)
                if disk_parts[2].isdigit:
                    disk_data['size'] = disk_parts[2]
                else:
                    msg = _('Volume SIZE is not a digit')
                    raise argparse.ArgumentTypeError(msg)
                config_attrs['disk'].append(disk_data)
            else:
                msg = _('Cannot parse disk information')
                raise argparse.ArgumentTypeError(msg)

        if parsed_args.public_ip_bandwith:
            ip_struct = {
                'eip': {
                    'ip_type': '5_bgp',
                    'bandwidth': {
                        'size': parsed_args.public_ip_bandwith,
                        'share_type': 'PER',
                        'charging_mode': 'traffic'
                    }
                }
            }
            config_attrs['public_ip'] = ip_struct

        if parsed_args.user_data:
            with open(parsed_args.user_data, 'r') as file:
                # Read the file (ASCII), encode Base64 and use that
                data = file.read().encode('ascii')
                data_b64 = base64.b64encode(data).decode('ascii')
                config_attrs['user_data'] = data_b64

        if parsed_args.metadata:
            config_attrs['metadata'] = {}
            k, v = parsed_args.metadata.split('=')
            config_attrs['metadata'][k] = v

        args = {}
        args['instance_config'] = config_attrs

        client = self.app.client_manager.auto_scaling

        instance = client.create_config(name=parsed_args.name, **args)

        fmt = set_attributes_for_print_detail(instance)
        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(
            fmt, self.columns, formatters={})

        return (self.columns, data)


class DeleteAutoScalingConfig(command.Command):
    _description = _('Deletes AutoScalinig group')

    def get_parser(self, prog_name):
        parser = super(DeleteAutoScalingConfig, self).get_parser(prog_name)
        parser.add_argument(
            'config',
            nargs='+',
            metavar='<config>',
            help=_('AS Configuration ID')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        # TODO(agoncharov) - proper error handling (reporting)

        if len(parsed_args.config) > 1:
            client.batch_delete_configs(parsed_args.config)
        elif len(parsed_args.config) == 1:
            client.delete_config(parsed_args.config[0], ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import _base


class TestBase(base.TestCase):
    def setUp(self):
        super(TestBase, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.delete = mock.Mock()
        self.sess.default_microversion = None
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self._translate_response = mock.Mock()

    def test_fetch(self):
        sot = _base.Resource.existing(id=1)
        sot._translate_response = mock.Mock()
        sot.allow_delete = True

        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {'job_id': 'fake'}
        response.headers = {}
        self.sess.delete.return_value = response

        # Restore from backup
        rt = sot.delete(self.sess)

        sot._translate_response.assert_called_with(response, has_body=True)
        self.assertIsInstance(rt, _base.Resource)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Template(resource.Resource):
    resources_key = 'message_templates'
    base_path = '/notifications/message_template'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'offset', 'limit', 'protocol', 'name',
        name='message_template_name')

    #: Unique Request ID
    id = resource.Body('message_template_id', alternate_id=True)
    #: Specifies the Template Name.
    #: Contains only digits, letters, underscores and hyphens.
    name = resource.Body('message_template_name')
    #: Protocol supported by the template.
    protocol = resource.Body('protocol')
    #: Template content, which currently supports plain text only.
    content = resource.Body('content')
    #: Template variable list.
    tag_names = resource.Body('tag_names', type=list)
    #: Time when the template was created.
    #:  The UTC time is in YYYY-MM-DDTHH:MM:SSZ format.
    create_time = resource.Body('create_time')
    #: Last time when the template was updated.
    #:  The UTC time is in YYYY-MM-DDTHH:MM:SSZ format.
    update_time = resource.Body('update_time')
    #: Request ID, which is unique.
    request_id = resource.Body('request_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#


def assert_attributes_equal(test_case, sot, data):
    if isinstance(data, dict):
        try:
            test_case.assertEqual(sot, data)
        except AssertionError:
            for k, v in data.items():
                if isinstance(v, list):
                    assert_attributes_equal(test_case, getattr(sot, k), v)
                else:
                    try:
                        test_case.assertEqual(getattr(sot, k), v)
                    except AssertionError:
                        assert_attributes_equal(test_case, getattr(sot, k), v)
    elif isinstance(data, list):
        try:
            test_case.assertEqual(sot, data)
        except AssertionError:
            for ix, list_data in enumerate(data):
                assert_attributes_equal(test_case, sot[ix], list_data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v1 import cluster as _cluster

OS_HEADERS = {
    'Content-Type': 'application/json',
}

EXAMPLE_LIST = [{
    'kind': 'cluster',
    'apiVersion': 'v1',
    'metadata': {
        'name': 'ag-test',
        'uuid': '5a66a449-668c-492f-8c33-5cdbdeaadd2e',
        'spaceuuid': 'de213bc4-4949-2a70-1a28-ae6c54e4ea23',
        'createAt': '2018-04-03 12:27:23.903327651 +0000 UTC',
        'updateAt': '2018-04-03 13:38:11.018224077 +0000 UTC'},
    'spec': {
        'description': 'some description',
        'hostList': {
            'kind': 'list',
            'apiVersion': 'v1',
            'metadata': {},
            'spec': {
                'hostList': [{
                    'kind': 'host',
                    'apiVersion': 'v1',
                    'metadata': {
                        'name': 'ag-test-node-1',
                        'spaceuuid': 'de213bc4-4949-2a70-1a28-ae6c54e4ea23',
                        'createAt': '2018-04-03 13:37:27.993379777 +0000 UTC',
                        'updateAt': '2018-04-03 13:37:27.993379777 +0000 UTC'},
                    'spec': {
                        'clusteruuid': '5a66a449-668c-492f-8c33-5cdbdeaadd2e',
                        'clustername': 'ag-test',
                        'flavor': 's1.medium',
                        'cpu': 1,
                        'memory': 4096,
                        'label': 'fake',
                        'status': {
                            'capacity': {
                                'cpu': '',
                                'memory': '',
                                'pods': ''
                            },
                            'allocatable': {
                                'cpu': '',
                                'memory': '',
                                'pods': ''
                            },
                            'conditions': None,
                            'addresses': None,
                            'daemonEndpoints': {
                                'kubeletEndpoint': {'Port': 0}
                            },
                            'nodeInfo': {
                                'machineID': '',
                                'systemUUID': '',
                                'bootID': '',
                                'kernelVersion': '',
                                'osImage': '',
                                'containerRuntimeVersion': '',
                                'kubeletVersion': '',
                                'kubeProxyVersion': ''},
                            'images': None
                        }
                    },
                    'replicas': 1,
                    'status': 'BUILD'
                }]
            }
        },
        'az': 'eu-de-01',
        'cpu': 1,
        'memory': 4096,
        'vpc': 'ag-vpc-636e',
        'vpcid': '4056b8b9-ff86-4cc0-8442-c75eba1034de',
        'subnet': 'ag-subnet-636e',
        'cidr': '192.168.0.0/24',
        'clustertype': 'Single',
        'latestversion': {
            'version': 'V1.2.0',
            'date': '2017-10-28 02:58:43.052591366 +0000 UTC',
            'detail': 'k8s-v1.7.3'},
        'security_group_id': '',
        'versioninfo': {
            'version': 'V1.2.0',
            'date': '2017-10-28 02:58:43.052591366 +0000 UTC',
            'detail': 'k8s-v1.7.3'
        }
    },
    'clusterStatus': {'status': 'DEPLOYING'},
    'k8s_version': '1.7.3',
    'commit_id': '2c2fe6e8278a5db2d15a013987b53968c743f2a1'}
]


class TestCluster(base.TestCase):

    def setUp(self):
        super(TestCluster, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.put = mock.Mock()

        self.sot = _cluster.Cluster()

        self.sot_expected = _cluster.Cluster(**EXAMPLE_LIST[0])

    def test_basic(self):
        sot = _cluster.Cluster()
        self.assertEqual('', sot.resource_key)
        self.assertEqual('', sot.resources_key)
        self.assertEqual('/clusters',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST[0]
        sot = _cluster.Cluster.existing(**obj)
        self.assertEqual(obj['metadata']['uuid'], sot.id)
        self.assertEqual(obj['kind'], sot.kind)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)

        self.sess.get.return_value = mock_response

        result = list(
            self.sot.list(
                self.sess,
            )
        )

        self.sess.get.assert_called_once_with(
            '/clusters',
            params={},
            headers=OS_HEADERS
        )

        expected_list = [
            self.sot_expected,
        ]

        self.assertEqual(expected_list, result)

    def test_get(self):
        sot = _cluster.Cluster.existing(id=EXAMPLE_LIST[0]['metadata']['uuid'])

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST[0])

        self.sess.get.return_value = mock_response

        result = sot.get(self.sess)

        self.sess.get.assert_called_once_with(
            'clusters/%s' % EXAMPLE_LIST[0]['metadata']['uuid'],
            headers=OS_HEADERS
        )

        self.assertDictEqual(
            result.to_dict(),
            self.sot_expected.to_dict()
        )

    def test_create(self):
        cluster = {
            'metadata': {
                'name': 'test',
            },
            'spec': {
                'description': 'descr',
                'vpc': 'vpc-id',
                'subnet': 'subnet-id',
                'region': 'regio',
                'security_group_id': 'sg_id',
                'clustertype': 'Single'
            }
        }
        sot = _cluster.Cluster.new(**cluster)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST[0])

        self.sess.post.return_value = mock_response

        result = sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        expected_json = copy.deepcopy(cluster)
        expected_json['kind'] = 'cluster'
        expected_json['apiVersion'] = 'v1'

        self.assertEqual('/clusters', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertDictEqual(
            result.to_dict(),
            self.sot_expected.to_dict())

    def test_create_name_normalize(self):
        """Ensure name passed in the root is properly mapped into metadata
        """
        cluster = {
            'name': 'test',
            'spec': {
                'description': 'descr',
                'vpc': 'vpc-id',
                'subnet': 'subnet-id',
                'region': 'regio',
                'security_group_id': 'sg_id',
                'clustertype': 'Single'
            }
        }
        sot = _cluster.Cluster.new(**cluster)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST[0])

        self.sess.post.return_value = mock_response

        sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        expected_json = copy.deepcopy(cluster)
        expected_json['kind'] = 'cluster'
        expected_json['apiVersion'] = 'v1'
        expected_json['metadata'] = {'name': 'test'}
        expected_json.pop('name')

        self.assertEqual('/clusters', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once()

    def test_update(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST[0])

        self.sess.put.return_value = mock_response

        sot = _cluster.Cluster.existing(**EXAMPLE_LIST[0])

        sot._update(**{'spec': {
            'publicip_id': 'some_ip',
            'description': 'descr'}})

        sot.update(self.sess, prepend_key=False)

        call_args = self.sess.put.call_args_list[0]

        self.assertEqual('clusters/' + sot.id, call_args[0][0])
        self.assertDictEqual(
            {'spec': {'publicip_id': 'some_ip', 'description': 'descr'}},
            call_args[1]['json'])

        self.sess.put.assert_called_once()

    def test_delete(self):
        sot = _cluster.Cluster.existing(id=EXAMPLE_LIST[0]['metadata']['uuid'])

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = None

        self.sess.delete.return_value = mock_response

        sot.delete(self.sess)

        self.sess.delete.assert_called_once_with(
            'clusters/%s' % sot.id,
            headers=OS_HEADERS)

    def test_delete_nodes(self):
        sot = _cluster.Cluster.existing(id=EXAMPLE_LIST[0]['metadata']['uuid'])

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = None

        self.sess.delete.return_value = mock_response

        sot.delete_nodes(self.sess, ['n1', 'n2'])

        self.sess.delete.assert_called_once_with(
            'clusters/%s/hosts' % sot.id,
            json={
                'hosts': [
                    {'name': 'n1'},
                    {'name': 'n2'},
                ]
            },
            headers=OS_HEADERS
        )

        self.sess.delete.reset_mock()
        sot.delete_nodes(self.sess, 'n3')

        self.sess.delete.assert_called_once_with(
            'clusters/%s/hosts' % sot.id,
            json={
                'hosts': [
                    {'name': 'n3'},
                ]
            },
            headers=OS_HEADERS
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.plas.v1 import _proxy

from openstack.tests.unit import test_proxy_base


class TestPlasProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestPlasProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''Anti DDoS Floating IP statistics v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import format

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListFloatingIPEvents(command.Lister):
    _description = _('List Anti DDoS FloatingIP events')
    columns = (
        'start_time', 'end_time', 'status', 'trigger_bps',
        'trigger_pps', 'trigger_http_pps'
    )

    def get_parser(self, prog_name):
        parser = super(ListFloatingIPEvents, self).get_parser(prog_name)

        parser.add_argument(
            'floating_ip',
            metavar='<UUID>',
            help=_('UUID of the floating IP.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.anti_ddos

        data = client.floating_ip_events(
            floating_ip_id=parsed_args.floating_ip)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns
                 ) for s in data))
        return table


class ListFloatingIPStatDay(command.Lister):
    _description = _('List Anti DDoS FloatingIP day statistics')
    columns = (
        'period_start', 'bps_in', 'bps_attack', 'total_bps',
        'pps_in', 'pps_attack', 'total_pps'
    )

    def get_parser(self, prog_name):
        parser = super(ListFloatingIPStatDay, self).get_parser(prog_name)

        parser.add_argument(
            'floating_ip',
            metavar='<UUID>',
            help=_('UUID of the floating IP.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.anti_ddos

        data = client.floating_ip_stat_day(
            floating_ip_id=parsed_args.floating_ip)

        table = (self.columns,
                 (utils.get_item_properties(s, self.columns)
                  for s in data))
        return table


class ListFloatingIPStatWeek(command.Lister):
    _description = _('List Anti DDoS FloatingIP week statistics')
    columns = (
        'period_start_date', 'ddos_intercept_times', 'ddos_blackhole_times',
        'max_attack_bps', 'max_attack_conns'
    )

    def get_parser(self, prog_name):
        parser = super(ListFloatingIPStatWeek, self).get_parser(prog_name)

        parser.add_argument(
            '--start_time',
            metavar='<yyyy-MM-ddTHH:mm:ss>',
            help=_('Start time of the period to be queried.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.anti_ddos

        query = {}

        if parsed_args.start_time:
            query['period_start_date'] = format.TimeTMsStr().serialize(
                parsed_args.start_time)

        data = client.floating_ip_stat_week(**query)

        table = (self.columns,
                 (utils.get_item_properties(s, self.columns)
                  for s in data.weekdata))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import instance

EXAMPLE = {
    "name": "test-cluster-01",
    "datastore": {
        "type": "DDS-Community",
        "version": "3.4",
        "storage_engine": "wiredTiger"
    },
    "region": "aaa",
    "availability_zone": "bbb",
    "vpc_id": "674e9b42-cd8d-4d25-a2e6-5abcc565b961",
    "subnet_id": "f1df08c5-71d1-406a-aff0-de435a51007b",
    "security_group_id": "7aa51dbf-5b63-40db-9724-dad3c4828b58",
    "password": "Test@123",
    "mode": "Sharding",
    "flavor": [
        {
            "type": "mongos",
            "num": 2,
            "spec_code": "dds.mongodb.s2.medium.4.mongos"
        },
        {
            "type": "shard",
            "num": 2,
            "storage": "ULTRAHIGH",
            "size": 20,
            "spec_code": "dds.mongodb.s2.medium.4.shard"
        },
        {
            "type": "config",
            "num": 1,
            "storage": "ULTRAHIGH",
            "size": 20,
            "spec_code": "dds.mongodb.s2.large.2.config"
        }
    ],
    "backup_strategy": {
        "start_time": "08:15-09:15",
        "keep_days": "8"
    },
    "ssl_option": "1"
}


class TestFlavor(base.TestCase):

    def test_basic(self):
        sot = instance.Instance()

        self.assertEqual('/instances', sot.base_path)
        self.assertEqual('instances', sot.resources_key)
        self.assertEqual('instance', sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)
        self.assertDictEqual({
            'id': 'id',
            'name': 'name',
            'mode': 'mode',
            'marker': 'marker',
            'datastore_type': 'datastore_type',
            'vpc_id': 'vpc_id',
            'subnet_id': 'subnet_id',
            'limit': 'limit',
            'offset': 'offset'},
            sot._query_mapping._mapping)

    def test_make_it(self):
        sot = instance.Instance(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['security_group_id'], sot.security_group_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.tests.functional.sdk.ctsv3 import TestCtsv3


class TestKeyEvent(TestCtsv3):
    uuid_v4 = uuid.uuid4().hex[:8]

    def test_01_create_event(self):
        attrs = {
            "notification_name": f'event_{self.uuid_v4}',
            "operation_type": "complete",
        }
        event = self.conn.ctsv3.create_key_event(**attrs)
        self.assertGreater(len(event['notification_id']), 0)

    def test_02_key_events(self):
        events = list(self.conn.ctsv3.key_events(notification_type='smn'))
        self.assertGreater(len(events), 0)

    def test_03_update_event(self):
        event = list(self.conn.ctsv3.key_events(
            notification_type='smn',
            notification_name=f'event_{self.uuid_v4}'))[0]
        attrs = {
            "notification_name": f'event_{self.uuid_v4}_update',
            "operation_type": "complete",
            "status": "disabled",
            "notification_id": event['notification_id']
        }
        event = self.conn.ctsv3.update_key_event(**attrs)
        self.assertGreater(len(event['notification_id']), 0)

    def test_04_delete_event(self):
        event = list(self.conn.ctsv3.key_events(
            notification_type='smn',
            notification_name=f'event_{self.uuid_v4}_update'))[0]
        self.conn.ctsv3.delete_key_event(event)
        events = list(self.conn.ctsv3.key_events(notification_type='smn'))
        test_event = [
            event for event in events
            if event['notification_name'] == f'event_{self.uuid_v4}_update']
        self.assertEqual(len(test_event), 0)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create new organization permissions
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

attrs = {
    'namespace': 'test',
    'permissions': [
        {
            "user_id": "5a23ecb3999b458d92d51d524bb7fb4c",
            "user_name": "test",
            "user_auth": 1
        }
    ],
}

perm = conn.swr.create_organization_permissions(**attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import argparse
import mock

from otcextensions.osclient.dns.v2 import zone
from otcextensions.tests.unit.osclient.dns.v2 import fakes


class TestListZone(fakes.TestDNS):

    objects = fakes.FakeZone.create_multiple(3)

    columns = (
        'id', 'name', 'zone_type', 'serial', 'status', 'action'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListZone, self).setUp()

        self.cmd = zone.ListZone(self.app, None)

        self.client.zones = mock.Mock()
        self.client.api_mock = self.client.zones

    def test_default(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_default_query(self):
        arglist = [
            '--type', 'private'
        ]

        verifylist = [
            ('type', 'private')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            zone_type='private'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowZone(fakes.TestDNS):

    _data = fakes.FakeZone.create_one()

    columns = (
        'action', 'created_at', 'description', 'email', 'id', 'name',
        'pool_id', 'record_num', 'router', 'serial', 'status', 'ttl',
        'updated_at', 'zone_type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowZone, self).setUp()

        self.cmd = zone.ShowZone(self.app, None)

        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.find_zone

    def test_default(self):
        arglist = [
            'zone'
        ]

        verifylist = [
            ('zone', 'zone')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            'zone',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateZone(fakes.TestDNS):

    _data = fakes.FakeZone.create_one()

    columns = (
        'action', 'created_at', 'description', 'email', 'id', 'name',
        'pool_id', 'record_num', 'router', 'serial', 'status', 'ttl',
        'updated_at', 'zone_type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateZone, self).setUp()

        self.cmd = zone.CreateZone(self.app, None)

        self.client.create_zone = mock.Mock()
        self.client.api_mock = self.client.create_zone

    def test_create(self):
        arglist = [
            'zn',
            '--email', 'eml',
            '--description', 'descr',
            '--type', 'public',
            '--ttl', '500',
            '--router_id', 'rid',
            '--router_region', 'regio',
        ]

        verifylist = [
            ('name', 'zn'),
            ('email', 'eml'),
            ('description', 'descr'),
            ('type', 'public'),
            ('ttl', 500),
            ('router_id', 'rid'),
            ('router_region', 'regio'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            description='descr',
            email='eml',
            name='zn',
            ttl=500,
            zone_type='public'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_private(self):
        arglist = [
            'zn',
            '--email', 'eml',
            '--description', 'descr',
            '--type', 'private',
            '--ttl', '500',
            '--router_id', 'rid',
            '--router_region', 'regio',
        ]

        verifylist = [
            ('name', 'zn'),
            ('email', 'eml'),
            ('description', 'descr'),
            ('type', 'private'),
            ('ttl', 500),
            ('router_id', 'rid'),
            ('router_region', 'regio'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            description='descr',
            email='eml',
            name='zn',
            ttl=500,
            zone_type='private',
            router={'router_id': 'rid', 'router_region': 'regio'}
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_private_raise_no_rid(self):
        arglist = [
            'zn',
            '--email', 'eml',
            '--description', 'descr',
            '--type', 'private',
            '--ttl', '500',
            '--router_region', 'regio',
        ]

        verifylist = [
            ('name', 'zn'),
            ('email', 'eml'),
            ('description', 'descr'),
            ('type', 'private'),
            ('ttl', 500),
            ('router_region', 'regio'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        self.assertRaises(argparse.ArgumentTypeError,
                          self.cmd.take_action, parsed_args)

        self.client.api_mock.assert_not_called()


class TestSetZone(fakes.TestDNS):

    _data = fakes.FakeZone.create_one()

    columns = (
        'action', 'created_at', 'description', 'email', 'id', 'name',
        'pool_id', 'record_num', 'router', 'serial', 'status', 'ttl',
        'updated_at', 'zone_type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestSetZone, self).setUp()

        self.cmd = zone.SetZone(self.app, None)

        self.client.update_zone = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.update_zone

    def test_update(self):
        arglist = [
            'zn',
            '--email', 'eml',
            '--description', 'descr',
            '--ttl', '500',
        ]

        verifylist = [
            ('zone', 'zn'),
            ('email', 'eml'),
            ('description', 'descr'),
            ('ttl', 500),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]
        self.client.find_zone.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._data,
            description='descr',
            email='eml',
            ttl=500
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteZone(fakes.TestDNS):

    def setUp(self):
        super(TestDeleteZone, self).setUp()

        self.cmd = zone.DeleteZone(self.app, None)

        self.client.delete_zone = mock.Mock()
        self.client.api_mock = self.client.delete_zone
        self.client.find_zone = mock.Mock()

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('zone', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [{}, {}]
        self.client.find_zone.side_effect = ['t1', 't2']

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(zone='t1'),
            mock.call(zone='t2')
        ]

        self.client.api_mock.assert_has_calls(calls)
        self.assertEqual(2, self.client.api_mock.call_count)


class TestAssociateRouter(fakes.TestDNS):

    _data = fakes.FakeZone.create_one()

    columns = (
        'action', 'created_at', 'description', 'email', 'id', 'name',
        'pool_id', 'record_num', 'router', 'serial', 'status', 'ttl',
        'updated_at', 'zone_type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestAssociateRouter, self).setUp()

        self.cmd = zone.AssociateRouterToZone(self.app, None)

        self.client.add_router_to_zone = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.add_router_to_zone

    def test_default(self):
        arglist = [
            'zn',
            '--router_id', 'rid',
            '--router_region', 'regio',
        ]

        verifylist = [
            ('zone', 'zn'),
            ('router_id', 'rid'),
            ('router_region', 'regio'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]
        self.client.find_zone.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._data,
            router_id='rid',
            router_region='regio'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDisassociateRouter(fakes.TestDNS):

    _data = fakes.FakeZone.create_one()

    columns = (
        'action', 'created_at', 'description', 'email', 'id', 'name',
        'pool_id', 'record_num', 'router', 'serial', 'status', 'ttl',
        'updated_at', 'zone_type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestDisassociateRouter, self).setUp()

        self.cmd = zone.DisassociateRouterToZone(self.app, None)

        self.client.remove_router_from_zone = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.remove_router_from_zone

    def test_default(self):
        arglist = [
            'zn',
            '--router_id', 'rid',
            '--router_region', 'regio',
        ]

        verifylist = [
            ('zone', 'zn'),
            ('router_id', 'rid'),
            ('router_region', 'regio'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]
        self.client.find_zone.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._data,
            router_id='rid',
            router_region='regio'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestListNameserver(fakes.TestDNS):

    objects = fakes.FakeNameserver.create_multiple(3)
    _zone = fakes.FakeZone.create_one()

    columns = (
        'address', 'hostname', 'priority'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListNameserver, self).setUp()

        self.cmd = zone.ListNameserver(self.app, None)

        self.client.nameservers = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.nameservers

    def test_default(self):
        arglist = [
            'zn'
        ]

        verifylist = [
            ('zone', 'zn')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]
        self.client.find_zone.side_effect = [
            self._zone
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._zone
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.nat.v2 import gateway as _gateway
from otcextensions.sdk.nat.v2 import snat as _snat
from otcextensions.sdk.nat.v2 import dnat as _dnat

from openstack import proxy


class Proxy(proxy.Proxy):

    skip_discovery = True

    # ======== Gateway ========
    def create_gateway(self, **attrs):
        """Create a new gateway from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`,
            comprised of the properties on the Gateway class.

        :returns: The results of the Gateway Creation

        :rtype: :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`
        """
        return self._create(_gateway.Gateway, **attrs)

    def delete_gateway(self, gateway, ignore_missing=True):
        """Delete a gateway

        :param gateway: The value can be the ID of a NAT Gatway or a
            :class:`~otcextensions.sdk.nat.v2.gateway.Gateway` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the gateway does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent gateway.

        :returns: ``None``
        """
        return self._delete(_gateway.Gateway, gateway,
                            ignore_missing=ignore_missing)

    def wait_for_gateway(self, gateway, status='ACTIVE', failures=None,
                         interval=2, wait=300, attribute='status'):
        """Wait for an gateway to be in a particular status.

        :param gateway:
            The :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`
            or gateway ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['INACTIVE'] if failures is None else failures
        return resource.wait_for_status(
            self, gateway, status, failures, interval, wait)

    def wait_for_delete_gateway(self, gateway, interval=2, wait=180):
        """Wait for the gateway to be deleted.

        :param gateway:
            The :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`
            or instance ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        gateway = self._get_resource(_gateway.Gateway, gateway)
        return resource.wait_for_delete(self, gateway, interval, wait)

    def gateways(self, **query):
        """Return a generator of gateways

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of gateway objects.
        """
        return self._list(_gateway.Gateway, **query)

    def get_gateway(self, gateway):
        """Get a single gateway

        :param gateway: The value can be the ID of a NAT Gatway or a
            :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`
            instance.

        :returns: One :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_gateway.Gateway, gateway)

    def update_gateway(self, gateway, **attrs):
        """Update a gateway

        :param gateway: The value can be either the ID of a gateway or a
            :class:`~otcextensions.sdk.nat.v2.gateway.Gateway` instance.
        :param dict attrs: The attributes to update on the gateway represented
            by ``gateway``.

        :returns: The updated gateway.

        :rtype: :class:`~otcextensions.sdk.nat.v2.gateway.Gateway`
        """
        return self._update(_gateway.Gateway, gateway, **attrs)

    def find_gateway(self, name_or_id, ignore_missing=False):
        """Find a single gateway

        :param name_or_id: The name or ID of a gateway
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the gateway does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent gateway.

        :returns:
            One :class:`~otcextensions.sdk.nat.v2.gateway.Gateway` or ``None``
        """
        return self._find(_gateway.Gateway, name_or_id,
                          ignore_missing=ignore_missing)

    # ======== SNAT rules ========
    def create_snat_rule(self, **attrs):
        """Create a new SNAT rule from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.nat.v2.snat.Snat`, comprised of the
            properties on the Snat class.

        :returns: The result of Snat rule creation

        :rtype: :class:`~otcextensions.sdk.nat.v2.snat.Snat`
        """
        return self._create(_snat.Snat, **attrs)

    def delete_snat_rule(self, snat, ignore_missing=True):
        """Delete a SNAT rule

        :param gateway: The value can be the ID of a snat rule or a
            :class:`~otcextensions.sdk.nat.v2.snat.Snat` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the snat rule does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent snat rule.

        :returns: ``None``
        """
        return self._delete(_snat.Snat, snat, ignore_missing=ignore_missing)

    def get_snat_rule(self, snat_rule):
        """Get a single SNAT rule

        :param snat_rule: The value can be the ID of a snat rule or a
            :class:`~otcextensions.sdk.nat.v2.snat.Snat` instance.

        :returns: One :class:`~otcextensions.sdk.nat.v2.snat.Snat`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_snat.Snat, snat_rule)

    def snat_rules(self, **query):
        """Return a generator of SNAT rules

        :param dict query: Optional query parameters to be sent to limit
            the snat rules being returned.

        :returns: A generator of Snat objects.
        """
        return self._list(_snat.Snat, **query)

    def wait_for_snat(self, snat, status='ACTIVE', failures=None,
                      interval=2, wait=300, attribute='status'):
        """Wait for an snat rule to be in a particular status.

        :param snat:
            The :class:`~otcextensions.sdk.nat.v2.snat.Snat`
            or snat ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['INACTIVE'] if failures is None else failures
        return resource.wait_for_status(
            self, snat, status, failures, interval, wait)

    def wait_for_delete_snat(self, snat, interval=2, wait=180):
        """Wait for the snat rule to be deleted.

        :param snat:
            The :class:`~otcextensions.sdk.nat.v2.snat.Snat`
            or instance ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        snat = self._get_resource(_snat.Snat, snat)
        return resource.wait_for_delete(self, snat, interval, wait)

    # ======== DNAT rules ========
    def create_dnat_rule(self, **attrs):
        """Create a new DNAT rule from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.nat.v2.dnat.Dnat`, comprised of the
            properties on the Dnat class.

        :returns: The result of Dnat rule creation.

        :rtype: :class:`~otcextensions.sdk.nat.v2.dnat.Dnat`
        """
        return self._create(_dnat.Dnat, **attrs)

    def delete_dnat_rule(self, dnat, ignore_missing=True):
        """Delete a DNAT rule

        :param gateway: The value can be the ID of a dnat rule or a
            :class:`~otcextensions.sdk.nat.v2.dnat.Dnat` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the dnat rule does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent dnat rule.

        :returns: ``None``
        """
        return self._delete(_dnat.Dnat, dnat, ignore_missing=ignore_missing)

    def get_dnat_rule(self, dnat_rule):
        """Get a single DNAT rule

        :param snat_rule: The value can be the ID of a dnat rule or a
            :class:`~otcextensions.sdk.nat.v2.dnat.Dnat` instance.

        :returns: One :class:`~otcextensions.sdk.nat.v2.dnat.Dnat`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_dnat.Dnat, dnat_rule)

    def dnat_rules(self, **query):
        """Return a generator of DNAT rules

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of Dnat objects.
        """
        return self._list(_dnat.Dnat, **query)

    def wait_for_dnat(self, dnat, status='ACTIVE', failures=None,
                      interval=2, wait=300, attribute='status'):
        """Wait for an dnat rule to be in a particular status.

        :param dnat:
            The :class:`~otcextensions.sdk.nat.v2.dnat.Dnat`
            or dnat ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['INACTIVE'] if failures is None else failures
        return resource.wait_for_status(
            self, dnat, status, failures, interval, wait)

    def wait_for_delete_dnat(self, dnat, interval=2, wait=180):
        """Wait for the dnat rule to be deleted.

        :param dnat:
            The :class:`~otcextensions.sdk.nat.v2.dnat.Dnat`
            or instance ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        dnat = self._get_resource(_dnat.Dnat, dnat)
        return resource.wait_for_delete(self, dnat, interval, wait)

    # ======== Project cleanup ========
    def _get_cleanup_dependencies(self):
        return {
            'nat': {
                'before': ['network']
            }
        }

    def _service_cleanup(self,
                         dry_run=True,
                         client_status_queue=None,
                         identified_resources=None,
                         filters=None,
                         resource_evaluation_fn=None,
                         skip_resources=None):
        if self.should_skip_resource_cleanup("gateway",
                                             skip_resources):
            return

        gateways = []
        for gateway in self.gateways():
            snat_rules = []
            for snat in self.snat_rules():
                need_delete = self._service_cleanup_del_res(
                    self.delete_snat_rule,
                    snat,
                    dry_run=dry_run,
                    client_status_queue=client_status_queue,
                    identified_resources=identified_resources,
                    filters=filters,
                    resource_evaluation_fn=resource_evaluation_fn)
                if not dry_run and need_delete:
                    snat_rules.append(snat)
            dnat_rules = []
            for dnat in self.dnat_rules():
                need_delete = self._service_cleanup_del_res(
                    self.delete_dnat_rule,
                    dnat,
                    dry_run=dry_run,
                    client_status_queue=client_status_queue,
                    identified_resources=identified_resources,
                    filters=filters,
                    resource_evaluation_fn=resource_evaluation_fn)
                if not dry_run and need_delete:
                    dnat_rules.append(dnat)
            for snat in snat_rules:
                self.wait_for_delete_snat(snat)
            for dnat in dnat_rules:
                self.wait_for_delete_dnat(dnat)
            need_delete = self._service_cleanup_del_res(
                self.delete_gateway,
                gateway,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)
            if dry_run and need_delete:
                for port in self._connection.network.ports(
                        device_id=gateway.id):
                    identified_resources[port.id] = port
            if not dry_run and need_delete:
                gateways.append(gateway)

        for gateway in gateways:
            self.wait_for_delete_gateway(gateway)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Create CBR policy with attributes
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'enabled': True,
    'name': 'my_policy',
    'operation_definition': {
        'day_backups': 0,
        'month_backups': 0,
        'max_backups': 1,
        'timezone': 'UTC+08:00',
        'week_backups': 0,
        'year_backups': 0
    },
    'operation_type': 'backup',
    'trigger': {
        'properties': {
            'pattern': [
                'FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR,SA,SU;BYHOUR=14;BYMINUTE=00'
            ]
        }
    }
}

policy = conn.cbr.create_policy(**attrs)
print(policy)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.vpc.v1 import vpc


IDENTIFIER = 'ID'
EXAMPLE = {
    "name": "vpc",
    "cidr": "192.168.200.0/24",
    "status": "CREATING",
    "description": "test",
    "tenant_id": "6fbe9263116a4b68818cf1edce16bc4f",
    "routes": [],
    "enable_shared_snat": False,
    "id": "3d42a0d4-a980-4613-ae76-a2cddecff054"
}


class TestVpc(base.TestCase):

    def setUp(self):
        super(TestVpc, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = vpc.Vpc()
        self.assertEqual('vpc', sot.resource_key)
        self.assertEqual('vpcs', sot.resources_key)
        path = '/v1/%(project_id)s/vpcs'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = vpc.Vpc(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['routes'], sot.routes)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['cidr'], sot.cidr)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['routes'], sot.routes)
        self.assertEqual(EXAMPLE['enable_shared_snat'], sot.enable_shared_snat)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestQueue(base.BaseFunctionalTest):
    QUEUE_ALIAS = 'sdk_test_queue'
    queues = []

    def setUp(self):
        super(TestQueue, self).setUp()
        try:
            self.queue = self.conn.dms.create_queue(
                name=TestQueue.QUEUE_ALIAS
            )
        except exceptions.DuplicateResource:
            self.queue = self.conn.dms.find_queue(alias=TestQueue.QUEUE_ALIAS)

        self.queues.append(self.queue)

    def tearDown(self):
        try:
            for queue in self.queues:
                if queue.id:
                    self.conn.dms.delete_queue(queue)
        except exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestQueue, self).tearDown()

    def test_list(self):
        self.queues = list(self.conn.dms.queues())
        self.assertGreaterEqual(len(self.queues), 0)
        if len(self.queues) > 0:
            queue = self.queues[0]
            q = self.conn.dms.get_queue(queue=queue.id)
            self.assertIsNotNone(q)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''Identity credential v3 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListCredentials(command.Lister):
    _description = _('List Identity Credentials')
    columns = (
        'access',
        'description',
        'user_id',
        'status',
        'created_at',
    )

    def get_parser(self, prog_name):
        parser = super(ListCredentials, self).get_parser(prog_name)

        parser.add_argument(
            '--user-id',
            metavar='<user-id>',
            help=_('User ID of the user using the credential')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.iam

        table_columns = (
            'Access Key',
            'Description',
            'User ID',
            'Status',
            'Created At',
        )

        attrs = {}

        if parsed_args.user_id:
            attrs['user_id'] = parsed_args.user_id

        data = client.credentials(**attrs)

        table = (table_columns,
                 (utils.get_dict_properties(
                     s, self.columns
                 ) for s in data))
        return table


class ShowCredential(command.ShowOne):
    _description = _('Show identity credential details')

    def get_parser(self, prog_name):
        parser = super(ShowCredential, self).get_parser(prog_name)

        parser.add_argument(
            'credential',
            metavar='<credential>',
            help=_('Access key of the credential.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.iam

        obj = client.find_credential(
            parsed_args.credential,
            ignore_missing=False
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteCredential(command.Command):
    _description = _('Delete identity credential')

    def get_parser(self, prog_name):
        parser = super(DeleteCredential, self).get_parser(prog_name)

        parser.add_argument(
            'credential',
            metavar='<credential>',
            nargs='+',
            help=_('Access key of the credential.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.credential:
            client = self.app.client_manager.iam
            for credential in parsed_args.credential:
                credential = client.find_credential(
                    credential,
                    ignore_missing=False)
                client.delete_credential(credential=credential)


class UpdateCredential(command.ShowOne):
    _description = _("Update identity credential.")

    def get_parser(self, prog_name):
        parser = super(UpdateCredential, self).get_parser(prog_name)
        parser.add_argument(
            'credential',
            metavar='<credential>',
            help=_("Specifies the access key / ID of the credential."),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the credential."),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Switch status of the credential.\n'
                   'active: Credential is active\n'
                   'inactive: Credential is inactive\n'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.iam
        args_list = [
            'description', 'status'
        ]
        attrs = {}
        for arg in args_list:
            if getattr(parsed_args, arg):
                attrs[arg] = getattr(parsed_args, arg)
        credential = client.find_credential(parsed_args.credential)

        obj = client.update_credential(credential.id, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateCredential(command.ShowOne):
    _description = _('Create a identity credential')

    def get_parser(self, prog_name):
        parser = super(CreateCredential, self).get_parser(prog_name)

        parser.add_argument(
            'user_id',
            metavar='<user-id>',
            help=_('User ID of the user who will use the credential')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of the alarm')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.iam
        attrs = {}

        attrs['user_id'] = parsed_args.user_id
        if parsed_args.description:
            attrs['description'] = parsed_args.description

        obj = client.create_credential(
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update function code
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

code_attrs = {
    'code_filename': 'index.zip',
    'code_type': 'inline',
    'func_code': {
        'file': 'UEsDBAoAAAAIAPQ1M1gNImPLrAAAAAEBAAAIAAAAaW5kZXgucHlN'
                'jtEOgjAMRd/5igVfxDAlxhjDo0S/wB+YrMgMdMvWGYnh390wEfrU'
                '3nvb0xXjG85qLRU+Sk8NP0UhUb3RltjTaUwkNKwVKDuwbA0vQMrD'
                'AhK8KSsTFsoCeYvsMw2xUkeCvKu0hLRk+6LIZ0u5s3BwPFwwUEEG'
                '/yo6B4vEXcshyBG+lb437kfNFpEWhATrQmqGTkYVH0Pit8FEdCqM'
                '6VQtSGncxYPpPz5O3fgFUEsBAh4DCgAAAAgA9DUzWA0iY8usAAAA'
                'AQEAAAgAAAAAAAAAAAAAAPMCAAAAAGluZGV4LnB5UEsFBgAAAAAB'
                'AAEANgAAANIAAAAAAA=='
    }
}
code = conn.functiongraph.update_function_code("func_urn", **code_attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.tests.functional.sdk.dds import TestDds

shared_data = {}


class TestInstance(TestDds):
    uuid_v4 = uuid.uuid4().hex[:8]
    network = None
    instance = None

    def setUp(self):
        super(TestInstance, self).setUp()

    def create_network(self):
        cidr = '192.168.0.0/16'
        ipv4 = 4
        router_name = 'sdk-dds-test-router-' + self.uuid_v4
        net_name = 'sdk-dds-test-net-' + self.uuid_v4
        subnet_name = 'sdk-dds-test-subnet-' + self.uuid_v4

        network = self.conn.network.create_network(name=net_name)
        self.assertEqual(net_name, network.name)
        net_id = network.id
        subnet = self.conn.network.create_subnet(
            name=subnet_name,
            ip_version=ipv4,
            network_id=net_id,
            cidr=cidr
        )
        self.assertEqual(subnet_name, subnet.name)
        subnet_id = subnet.id

        router = self.conn.network.create_router(name=router_name)
        self.assertEqual(router_name, router.name)
        router_id = router.id
        interface = router.add_interface(
            self.conn.network,
            subnet_id=subnet_id
        )
        self.assertEqual(interface['subnet_id'], subnet_id)
        self.assertIn('port_id', interface)
        return {
            'router_id': router_id,
            'subnet_id': subnet_id,
            'network_id': net_id
        }

    def destroy_network(self, params: dict):
        router_id = params.get('router_id')
        subnet_id = params.get('subnet_id')
        network_id = params.get('network_id')
        router = self.conn.network.get_router(router_id)

        interface = router.remove_interface(
            self.conn.network,
            subnet_id=subnet_id
        )
        self.assertEqual(interface['subnet_id'], subnet_id)
        self.assertIn('port_id', interface)
        sot = self.conn.network.delete_router(
            router_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            subnet_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            network_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)

    def test_01_create_instance(self):
        print('Creating instance')
        security_group = self.net_client.find_security_group(
            name_or_id='default', ignore_missing=False)
        self.network = self.create_network()
        shared_data['network'] = self.network
        name = 'test-dds-' + self.uuid_v4
        datastore = {
            'type': 'DDS-Community',
            'version': '3.4',
            'storage_engine': 'wiredTiger'
        }
        region = 'eu-de'
        availability_zone = 'eu-de-01'
        vpc_id = self.network['router_id']
        subnet_id = self.network['network_id']
        security_group_id = security_group.id
        password = 'Test@123!'
        mode = 'Sharding'
        flavor = [
            {
                "type": "mongos",
                "num": 2,
                "spec_code": "dds.mongodb.s2.medium.4.mongos"
            },
            {
                "type": "shard",
                "num": 2,
                "storage": "ULTRAHIGH",
                "size": 20,
                "spec_code": "dds.mongodb.s2.medium.4.shard"
            },
            {
                "type": "config",
                "num": 1,
                "storage": "ULTRAHIGH",
                "size": 20,
                "spec_code": "dds.mongodb.s2.large.2.config"
            }
        ]
        backup_strategy = {
            'start_time': '23:00-00:00',
            'keep_days': '8'
        }
        self.instance = (
            self.client.create_instance(
                name=name,
                datastore=datastore,
                region=region,
                availability_zone=availability_zone,
                vpc_id=vpc_id,
                subnet_id=subnet_id,
                security_group_id=security_group_id,
                password=password,
                mode=mode,
                flavor=flavor,
                backup_strategy=backup_strategy
            )
        )
        shared_data['id'] = self.instance['id']
        print('Job for creation set')
        print('Waiting for job to finish')
        self.client.wait_job(self.instance['job_id'])
        print('Instance created')
        self.assertIsNotNone(self.instance)

    def test_02_list_instances(self):
        instances = list(self.client.instances())
        self.assertIsNotNone(instances)

    def test_03_get_instance(self):
        instance = self.client.get_instance(shared_data['id'])
        self.assertIsNotNone(instance)

    def test_04_find_instance(self):
        instance = self.client.find_instance(name_or_id=shared_data['id'])
        self.assertIsNotNone(instance)

    def test_05_bind_eip(self):
        instance = self.client.wait_normal_instance(shared_data['id'])
        group_id = [item for item in instance['groups']
                    if item['type'] == 'mongos'][0]['nodes'][0]['id']
        ip = [item for item in list(self.conn.network.ips())
              if item['status'] == 'DOWN'][0]
        eip = self.client.bind_eip(group_id,
                                   ip['floating_ip_address'],
                                   ip['id'])
        shared_data['eip'] = eip
        self.client.wait_job(eip['job_id'])
        self.assertIsNotNone(eip['job_id'])

    def test_06_unbind_eip(self):
        eip = self.client.unbind_eip(shared_data['eip']['node_id'])
        self.client.wait_job(eip['job_id'])
        self.assertIsNotNone(eip['job_id'])

    def test_07_enlarge_instance(self):
        print('Enlarging instance')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        group_id = [item for item in instance['groups']
                    if item['type'] == 'shard'][0]['id']
        result = self.client.enlarge_instance(shared_data['id'],
                                              size=60,
                                              group_id=group_id)
        print('Enlarging instance, job created')
        self.assertIsNotNone(result['job_id'])
        self.client.wait_job(result['job_id'])
        print('Enlarging instance finished')

    def test_06_adding_nodes(self):
        print('Adding nodes')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        attrs = {
            "node_type": "mongos",
            "spec_code": "dds.mongodb.s2.medium.4.mongos",
            "num": 1
        }
        result = self.client.add_instance_nodes(shared_data['id'], **attrs)
        print('Adding nodes, job created')
        self.assertIsNotNone(result['job_id'])
        self.client.wait_job(result['job_id'])
        print('Adding nodes finished')

    def test_07_modify_instance_specs(self):
        print('Modify instance specs')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        group_id = [item for item in instance['groups']
                    if item['type'] == 'mongos'][0]['nodes'][0]['id']
        attrs = {
            "spec_code": "dds.mongodb.s2.xlarge.4.mongos",
            "target_type": "mongos",
            "target_id": group_id
        }
        result = self.client.resize_instance(shared_data['id'], **attrs)
        print('Modify instance specs, job created')
        self.assertIsNotNone(result['job_id'])
        self.client.wait_job(result['job_id'])
        print('Modify instance specs finished')

    # def test_08_switchover(self):
    #     self.wait_job_to_finish()
    #     instance = self.client.switchover_instance(shared_data['id'])
    #     self.assertIsNot(shared_data['instance']['job_id'],
    #                      instance['job_id'])
    #     shared_data['instance']['job_id'] = instance['job_id']

    def test_09_enable_ssl(self):
        print('Enabling SSL')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        result = self.client.enable_instance_ssl(shared_data['id'])
        print('Enabling SSL, job created')
        self.assertIsNotNone(result['job_id'])
        self.client.wait_job(result['job_id'])
        print('Enabling SSL finished')

    def test_10_change_name(self):
        print('Changing name')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        new_name = 'new_name'
        self.client.change_instance_name(shared_data['id'], name=new_name)
        instance = self.client.get_instance(shared_data['id'])
        self.assertEqual(new_name, instance['name'])
        print('Changing name finished')

    def test_11_change_port(self):
        print('Changing port')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        result = self.client.change_instance_port(shared_data['id'], port=8888)
        print('Changing port, job created')
        self.assertIsNotNone(result['job_id'])
        self.client.wait_job(result['job_id'])
        print('Changing port finished')

    def test_12_change_security_group(self):
        print('Changing security group')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        security_group = self.net_client.find_security_group(
            name_or_id='default', ignore_missing=False)
        sec_id = security_group['id']
        instance = (self.client.
                    change_instance_security_group(shared_data['id'],
                                                   security_group_id=sec_id))
        print('Changing security group, job created')
        print(instance['job_id'])
        self.assertIsNotNone(instance['job_id'])
        self.client.wait_job(instance['job_id'])
        print('Changing security group finished')

    def test_13_change_ip(self):
        print('Changing ip')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        group_id = [item for item in instance['groups']
                    if item['type'] == 'mongos'][0]['nodes'][0]['id']
        attrs = {
            "node_id": group_id,
            "new_ip": "192.168.0.133"
        }
        instance = self.client.change_instance_private_ip(shared_data['id'],
                                                          **attrs)
        print('Changing ip, job created')
        self.assertIsNotNone(instance['job_id'])
        self.client.wait_job(instance['job_id'])
        print('Changing ip finished')

    def test_14_create_ip(self):
        print('Creating ip')
        attrs = {
            "dds_type": "shard",
            "password": "New_pass23@"
        }
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        instance = self.client.create_instance_ip(shared_data['id'], **attrs)
        print('Creating ip, job created')
        self.assertIsNotNone(instance)
        self.client.wait_job(instance['job_id'])
        print('Creating ip finished')

    def test_15_replica(self):
        self.client.wait_normal_instance(shared_data['instance']['id'])
        instance = (self.client.
                    configure_client_network(shared_data['id'],
                                             network_ranges=["192.168.0.0/16"])
                    )
        self.assertIsNotNone(instance['job_id'])
        self.client.wait_job(instance['job_id'])

    def test_16_restart_instance(self):
        print('Restarting instance')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        instance = self.client.restart_instance(shared_data['id'])
        print('Restarting instance, job created')
        self.assertIsNotNone(instance['job_id'])
        self.client.wait_job(instance['job_id'])
        print('Restarting instance finished')

    def test_17_delete_instance(self):
        print('Deleting instance')
        print('Check for the state')
        instance = self.client.wait_normal_instance(shared_data['id'])
        print(instance['status'])
        instance = None
        try:
            instance = self.client.delete_instance(instance=shared_data['id'])
        except Exception:
            self.destroy_network(shared_data['network'])
        self.assertIsNotNone(instance)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
 Create virtual interface
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

attrs = {
    "direct_connect_id": "direct_connect_uuid",
    "vgw_id": "virtual_gateway_uuid",
    "type": "public",
    "service_type": "vpc",
    "vlan": 2556,
    "bandwidth": 10,
    "local_gateway_v4_ip": "217.150.159.57/30",
    "remote_gateway_v4_ip": "217.150.159.58/30",
    "route_mode": "static",
    "remote_ep_group_id": "remote_ep_group_uuid"
}

vi = conn.dcaas.create_virtual_interface(**attrs)
print(vi)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Cluster Nodes v1 action implementations'''
import argparse
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_cluster_node(obj):
    """Flatten the structure of the cluster node into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'cluster_uuid': obj.spec.cluster_uuid,
        'private_ip': obj.spec.private_ip,
        'public_ip': obj.spec.public_ip,
        'status': obj.status,
        'memory': obj.spec.memory,
        'cpu': obj.spec.cpu,
        'flavor': obj.spec.flavor,
        'availability_zone': obj.spec.availability_zone,
        'ssh_key': obj.spec.ssh_key,
        'replica_count': obj.replica_count,
        'volumes': ';'.join(
            (x.disk_type + ':' + str(x.disk_size))
            for x in obj.spec.volume
        ) if obj.spec.volume else '',
        'conditions': ';'.join(
            (cond['type'] + '=' + cond['status'])
            for cond in obj.spec.status.conditions
        ) if obj.spec.status.conditions else ''
    }
    if obj.spec.status.capacity:
        cap_max = {
            'cpu_max': obj.spec.status.capacity.cpu,
            'mem_max': obj.spec.status.capacity.memory,
            'pods_max': obj.spec.status.capacity.pods,
        }
        data.update(cap_max)
    if obj.spec.status.allocatable:
        cap_alloc = {
            'cpu_allocatable': obj.spec.status.allocatable.cpu,
            'mem_allocatable': obj.spec.status.allocatable.memory,
            'pods_allocatable': obj.spec.status.allocatable.pods,
        }
        data.update(cap_alloc)

    return data


class ListCCEClusterNode(command.Lister):
    _description = _('List CCE Cluster Nodes')
    columns = ('ID', 'name', 'private_ip', 'public_ip',
               'availability_zone', 'status')

    def get_parser(self, prog_name):
        parser = super(ListCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The ID or name of the cluster.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        data = client.cluster_nodes(parsed_args.cluster)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_cluster_node(s), self.columns,
                 ) for s in data))
        return table


class ShowCCEClusterNode(command.ShowOne):
    _description = _('Show single Cluster node details')
    columns = ('ID', 'name', 'cluster_uuid',
               'flavor', 'cpu', 'memory', 'volumes',
               'cpu_max', 'mem_max', 'pods_max',
               'cpu_allocatable', 'mem_allocatable', 'pods_allocatable',
               'conditions',
               'private_ip', 'public_ip', 'availability_zone', 'ssh_key',
               'status', 'replica_count')

    def get_parser(self, prog_name):
        parser = super(ShowCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'node',
            metavar='<node>',
            help=_('ID of the cluster node.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        obj = client.find_cluster_node(
            cluster=parsed_args.cluster,
            node=parsed_args.node
        )

        data = utils.get_dict_properties(
            _flatten_cluster_node(obj), self.columns)

        return (self.columns, data)


class DeleteCCEClusterNode(command.Command):
    _description = _('Delete CCE Cluster nodes')

    def get_parser(self, prog_name):
        parser = super(DeleteCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'node',
            metavar='<node>',
            nargs='+',
            help=_('Name of the cluster node.')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.cluster and parsed_args.node:
            client = self.app.client_manager.cce
            client.delete_cluster_nodes(
                cluster=parsed_args.cluster,
                node_names=parsed_args.node)


class CreateCCEClusterNode(command.Command):
    _description = _('Create CCE Cluster Node')

    columns = ('ID', 'name', 'cluster_uuid',
               'flavor', 'cpu', 'memory', 'volumes',
               'cpu_max', 'mem_max', 'pods_max',
               'cpu_allocatable', 'mem_allocatable', 'pods_allocatable',
               'conditions',
               'private_ip', 'public_ip', 'availability_zone', 'ssh_key',
               'status', 'replica_count')

    def get_parser(self, prog_name):
        parser = super(CreateCCEClusterNode, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('Name or ID of the cluster.')
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('The flavor for the server.')
        )
        parser.add_argument(
            '--label',
            metavar='<label>',
            help=_('The label attached to the node.')
        )
        parser.add_argument(
            '--volume',
            metavar='<volume>',
            action='append',
            required=True,
            help=_(
                'Disk information to attach to the instance.\n'
                'format = `DISK_TYPE`,`VOLUME_TYPE`,`SIZE`\n'
                '`DISK_TYPE` can be in [SYS, DATA] and identifies '
                'whether disk should be a system or data disk\n'
                '`VOLUME_TYPE` can be in: \n'
                '* `SATA` = Common I/O \n'
                '* `SAS` = High I/O \n'
                '* `SSD` = Ultra-High I/O \n'
                '`SIZE` is size in Gb (40 hardcoded for root, [100..32768])\n'
                '(Repeat multiple times for multiple disks).')
        )
        parser.add_argument(
            '--ssh_key',
            metavar='<ssh_key>',
            required=True,
            help=_('SSH Key used to create the node.')
        )
        parser.add_argument(
            '--assign_floating_ip',
            required=True,
            action='store_true',
            help=_('Whether to assign floating IP to the server or not.')
        )
        parser.add_argument(
            '--availability_zone',
            metavar='<availability_zone>',
            help=_('Availability zone to place server in.')
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('Tag to assign to the server in KEY=VALUE format. '
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--replica_count',
            metavar='[1..15]',
            type=int,
            choices=range(1, 15),
            default=1,
            help=_('Number of node instances to create '
                   '(max 15 in the cluster).')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        spec = {}
        spec['flavor'] = parsed_args.flavor
        spec['ssh_key'] = parsed_args.ssh_key
        spec['availability_zone'] = parsed_args.availability_zone
        if parsed_args.label:
            spec['label'] = parsed_args.label
        if parsed_args.assign_floating_ip:
            spec['assign_floating_ip'] = parsed_args.assign_floating_ip
        attrs['replica_count'] = parsed_args.replica_count
        spec['volume'] = []
        for disk in parsed_args.volume:
            disk_parts = disk.split(',')
            disk_data = {}
            if 3 == len(disk_parts):
                dt = disk_parts[0].upper()
                # Disk type: SYS->root; DATA->data
                if dt in ('SYS', 'DATA'):
                    disk_data['disk_type'] = 'data' if dt == 'DATA' else 'root'
                else:
                    msg = _('Disk Type is not in (SYS, DATA)')
                    raise argparse.ArgumentTypeError(msg)
                vt = disk_parts[1].upper()
                if vt in ('SATA', 'SAS', 'SSD'):
                    disk_data['volume_type'] = vt
                else:
                    msg = _('Volume Type is not in (SATA, SAS, SSD)')
                    raise argparse.ArgumentTypeError(msg)
                if dt == 'DATA':
                    # Size only relevant for data disk
                    if disk_parts[2].isdigit:
                        disk_data['disk_size'] = disk_parts[2]
                    else:
                        msg = _('Volume SIZE is not a digit')
                        raise argparse.ArgumentTypeError(msg)
                spec['volume'].append(disk_data)
            else:
                msg = _('Cannot parse disk information')
                raise argparse.ArgumentTypeError(msg)
        if parsed_args.tag:
            tags = []
            for tag in parsed_args.tag:
                (k, v) = tag.split('=')
                tags.append(k + '.' + v)
            spec['tags'] = tags

        attrs['spec'] = spec

        client = self.app.client_manager.cce

        obj = client.add_node(cluster=parsed_args.cluster, **attrs)

        data = utils.get_dict_properties(
            _flatten_cluster_node(obj),
            self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.auto_scaling.v1 import _base


class Activity(_base.Resource):
    # resource_key = 'scaling_activity_log'
    resources_key = 'scaling_activity_log'
    base_path = '/scaling_activity_log/%(scaling_group_id)s'
    query_marker_key = 'start_number'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'start_time', 'end_time', 'limit', 'marker',
        marker=query_marker_key
    )

    #: Properties
    id = resource.Body('id')
    #: AutoScaling Group Reference this activity belongs to
    scaling_group_id = resource.URI('scaling_group_id')
    #: AutoScaling Activity status
    #: valid values include: ``SUCCESS``, ``FAIL``, ``DOING``
    status = resource.Body('status')
    #: UTC date and time of activity begin
    start_time = resource.Body('start_time')
    #: UTC date and time of activity finish
    end_time = resource.Body('end_time')
    #: AutoScaling Activity description
    description = resource.Body('description')
    #: changed instance number during the AutoScaling Activity
    scaling_value = resource.Body('scaling_value')
    #: current instance number during the AutoScaling Activity
    instance_value = resource.Body('instance_value', type=int)
    #: desired instance number of the AutoScaling Activity
    desire_value = resource.Body('desire_value', type=int)
    #: The instance list removed in the AutoScaling Activity
    instance_removed_list = resource.Body('instance_removed_list')
    #: The instance list deleted in the AutoScaling Activity
    instance_deleted_list = resource.Body('instance_deleted_list')
    #: The instance list added in the AutoScaling Activity
    instance_added_list = resource.Body('instance_added_list')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Pool(resource.Resource):
    resource_key = 'pool'
    resources_key = 'pools'
    base_path = '/elb/pools'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'description', 'healthmonitor_id',
        'id', 'name', 'loadbalancer_id', 'protocol',
        'lb_algorithm', 'enterprise_project_id',
        'ip_version', 'member_address', 'member_device_id',
        is_admin_state_up='admin_state_up'
    )

    # Properties
    #: Description.
    description = resource.Body('description')
    #: Specifies the ID of the health check configured
    #: for the backend server group.
    healthmonitor_id = resource.Body('healthmonitor_id')
    #: Specifies the administrative status of the backend server group.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the IP version supported by the backend server group.
    ip_version = resource.Body('ip_version')
    #: Specifies the load balancing algorithm used by the load balancer
    #: to route requests to backend servers.
    lb_algorithm = resource.Body('lb_algorithm')
    #: Specifies the ID of the listener associated with the
    #: backend server group.
    listener_id = resource.Body('listener_id')
    #: Lists the listeners associated with the backend server group.
    listeners = resource.Body('listeners', type=list)
    #: Specifies the ID of the associated load balancer.
    loadbalancer_id = resource.Body('loadbalancer_id')
    #: Lists the IDs of load balancers associated with the
    #: backend server group.
    loadbalancers = resource.Body('loadbalancers', type=list)
    #: Lists the backend servers in the backend server group.
    members = resource.Body('members', type=list)
    #: Specifies the project ID.
    project_id = resource.Body('project_id')
    #: Specifies the protocol used by the backend server group
    #: to receive requests. TCP, UDP, and HTTP are supported.
    protocol = resource.Body('protocol')
    #: Specifies whether to enable sticky sessions.
    session_persistence = resource.Body('session_persistence', type=dict)
    #: Specifies whether to enable slow start.
    slow_start = resource.Body('slow_start', type=dict)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts visualization job v1 action implementations"""
import logging

from cliff import columns as cliff_columns
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    "created_at": cli_utils.UnixTimestampFormatter,
    "config": cli_utils.YamlFormat,
}


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class JobStatus(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "UNKNOWN",
        1: "INIT",
        2: "IMAGE_CREATING",
        3: "IMAGE_FAILED",
        4: "SUBMIT_TRYING",
        5: "SUBMIT_FAILED",
        6: "DELETE_FAILED",
        7: "WAITING",
        8: "RUNNING",
        9: "KILLING",
        10: "COMPLETED",
        11: "FAILED",
        12: "KILLED",
        13: "CANCELED",
        14: "LOST",
        15: "SCALING",
        16: "SUBMIT_MODEL_FAILED",
        17: "DEPLOY_SERVICE_FAILED",
        18: "CHECK_INIT",
        19: "CHECK_RUNNING",
        20: "RUNNING_COMPLETED",
        21: "CHECK_FAILED",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class ListVisualizationJobs(command.Lister):
    _description = _(
        "Query the visualization jobs that meet the search criteria."
    )
    columns = (
        "Job Id",
        "Job Name",
        "Created At",
    )

    def get_parser(self, prog_name):
        parser = super(ListVisualizationJobs, self).get_parser(prog_name)

        sort_by_choices = [
            "job_id",
            "job_name",
            "job_desc",
            "status",
            "create_time",
        ]

        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Number of jobs displayed on each page. The value "
                "range is [1, 1000]. Default value: 10."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Index of the page to be queried. Default value: 1"),
        )
        parser.add_argument(
            "--status",
            choices=list(JobStatus.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Job status. The options are as follows:\n" + JobStatus.STR
            ),
        )
        parser.add_argument(
            "--sort-by",
            metavar="{" + ",".join(sort_by_choices) + "}",
            type=lambda s: s.lower(),
            choices=sort_by_choices,
            help=_("Sorting field. Default value: job_name"),
        )
        parser.add_argument(
            "--order",
            metavar="{asc, desc}",
            type=lambda s: s.lower(),
            choices=["asc", "desc"],
            help=_("Sorting order. Default value: desc"),
        )
        parser.add_argument(
            "--search-content",
            metavar="<search_content>",
            help=_("Search content, for example, a training job name."),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_("Workspace ID."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        attrs_list = (
            "limit",
            "offset",
            "status",
            "sort_by",
            "order",
            "search_content",
            "workspace_id",
        )
        query_params = {}
        for arg in attrs_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        data = client.visualization_jobs(**query_params)
        _formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
            "Status": JobStatus,
        }
        table = (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in data
            ),
        )
        return table


class CreateVisualizationJob(command.ShowOne):
    _description = _("Create a Visualization Job model")

    def get_parser(self, prog_name):
        parser = super(CreateVisualizationJob, self).get_parser(prog_name)

        parser.add_argument(
            "name",
            metavar="<name>",
            help=_("Name of a visualization job."),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            dest="job_desc",
            help=_("Description of a visualization job."),
        )
        parser.add_argument(
            "--train-url",
            metavar="<train_url>",
            required=True,
            help=_("OBS path"),
        )
        parser.add_argument(
            "--job-type",
            metavar="<job_type>",
            help=_("Type of a visualization job"),
        )
        parser.add_argument(
            "--flavor",
            metavar="<flavor>",
            help=_("Specifications when a visualization job is created."),
        )
        parser.add_argument(
            "--auto-stop-duration",
            metavar="<auto_stop_duration>",
            type=int,
            help=_("Auto stop duration. The value ranges from 0 to 24."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        attrs = {
            "job_name": parsed_args.name,
            "train_url": parsed_args.train_url,
        }

        if parsed_args.job_desc:
            attrs["job_desc"] = parsed_args.job_desc
        if parsed_args.job_type:
            attrs["job_type"] = parsed_args.job_type
        if parsed_args.flavor:
            attrs["flavor"] = {"code": parsed_args.flavor}

        if parsed_args.auto_stop_duration:
            attrs["schedule"] = {
                "type": "stop",
                "time_unit": "HOURS",
                "duration": parsed_args.auto_stop_duration,
            }

        obj = client.create_visualization_job(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)


class ShowVisualizationJob(command.ShowOne):
    _description = _("Show details of a visualization job")

    def get_parser(self, prog_name):
        parser = super(ShowVisualizationJob, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a visualization job"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        data = client.get_visualization_job(parsed_args.jobId)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return (display_columns, data)


class UpdateVisualizationJob(command.ShowOne):
    _description = _("Modify Visualization Job Description")

    def get_parser(self, prog_name):
        parser = super(UpdateVisualizationJob, self).get_parser(prog_name)

        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a visualization job"),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            required=True,
            help=_("Description of a visualization job"),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        obj = client.update_visualization_job(
            parsed_args.jobId, parsed_args.description
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)


class StopVisualizationJob(command.ShowOne):
    _description = _("Stop Visualization Job")

    def get_parser(self, prog_name):
        parser = super(StopVisualizationJob, self).get_parser(prog_name)

        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a visualization job"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        obj = client.stop_visualization_job(parsed_args.jobId)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)


class RestartVisualizationJob(command.ShowOne):
    _description = _("Restart Visualization Job")

    def get_parser(self, prog_name):
        parser = super(RestartVisualizationJob, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<jobId>",
            help=_("ID of a visualization job"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        data = client.restart_visualization_job(parsed_args.jobId)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return (display_columns, data)


class DeleteVisualizationJob(command.Command):
    _description = _("Delete ModelArts visualization Job(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteVisualizationJob, self).get_parser(prog_name)
        parser.add_argument(
            "jobId",
            metavar="<JobId>",
            nargs="+",
            type=int,
            help=_("ID of the ModelArts visualization job(s) to be deleted."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        result = 0
        for job_id in parsed_args.jobId:
            try:
                client.delete_visualization_job(job_id)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete visualization job(s) with "
                        "ID '%(job_id)s': %(e)s"
                    ),
                    {"job_id": job_id, "e": e},
                )
        if result > 0:
            total = len(parsed_args.jobId)
            msg = _(
                "%(result)s of %(total)s visualization job(s) failed "
                "to delete."
            ) % {
                "result": result,
                "total": total,
            }
            raise exceptions.CommandError(msg)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.auto_scaling.v1 import quota
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingQuota(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingQuota, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingQuota(TestAutoScalingQuota):

    quotas = fakes.FakeQuota.create_multiple(3)

    columns = ('type', 'used', 'quota', 'max')

    data = []

    for s in quotas:
        data.append((
            s.type,
            s.used,
            s.quota,
            s.max
        ))

    def setUp(self):
        super(TestListAutoScalingQuota, self).setUp()

        self.cmd = quota.ListAutoScalingQuota(self.app, None)

        self.client.quotas = mock.Mock()

    def test_list_quota(self):
        arglist = [
        ]

        verifylist = [
            ('group', None),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.quotas.side_effect = [
            self.quotas
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.quotas.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_group_quota(self):
        arglist = [
            '--group', 'grp',
        ]

        verifylist = [
            ('group', 'grp'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.quotas.side_effect = [
            self.quotas
        ]

        grp_mock = mock.Mock()
        grp_mock.id = 2

        self.client.find_group = mock.Mock(return_value=grp_mock)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.quotas.assert_called_once_with(group=2)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.sdk.dcs.v1 import _proxy
from otcextensions.sdk.dcs.v1 import backup as _backup
from otcextensions.sdk.dcs.v1 import config as _config
from otcextensions.sdk.dcs.v1 import instance as _instance
from otcextensions.sdk.dcs.v1 import restore_record as _restore_record
from otcextensions.sdk.dcs.v1 import statistic as _stat
from otcextensions.sdk.dcs.v1 import quota as _quota
from otcextensions.sdk.dcs.v1 import maintenance_time_window as _mtw
from otcextensions.sdk.dcs.v1 import service_specification as _service_spec
from otcextensions.sdk.dcs.v1 import availability_zone as _az

from openstack.tests.unit import test_proxy_base


class TestDCSProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestDCSProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_create_instance(self):
        self.verify_create(
            self.proxy.create_instance, _instance.Instance,
        )

    def test_instances(self):
        self.verify_list(
            self.proxy.instances, _instance.Instance,
        )

    def test_instances_query(self):
        self.verify_list(
            self.proxy.instances, _instance.Instance,
            method_kwargs={
                'start': '1',
                'limit': '2',
                'name': '3',
                'status': '4',
                'includeFailure': True,
                'exactMatchName': True
            },
            expected_kwargs={
                'start': '1',
                'limit': '2',
                'name': '3',
                'status': '4',
                'includeFailure': True,
                'exactMatchName': True
            }
        )

    def test_get_instance(self):
        self.verify_get(
            self.proxy.get_instance, _instance.Instance,
        )

    def test_find_instance(self):
        self.verify_find(
            self.proxy.find_instance, _instance.Instance,
        )

    def test_update_instance(self):
        self.sot = _instance.Instance()
        self.sot.update = mock.Mock(return_value=self.sot)
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.update_instance('VALUE', a='b')
        self.proxy._get_resource.assert_called_with(
            _instance.Instance,
            'VALUE',
            a='b'
        )

    def test_extend_instance(self):
        self.sot = _instance.Instance()
        self.sot.extend = mock.Mock(return_value={})
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.extend_instance(self.sot, 4)
        self.proxy._get_resource.assert_called_with(
            _instance.Instance,
            self.sot
        )
        self.sot.extend.assert_called_with(
            self.proxy,
            4
        )
        self.proxy._get.assert_called_with(
            _instance.Instance,
            self.sot
        )

    def test_delete_instance(self):
        self.verify_delete(
            self.proxy.delete_instance, _instance.Instance, True,
        )

    def test_stop_instance(self):
        self.sot = _instance.Instance()
        self.sot.stop = mock.Mock(return_value={})
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._find = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.stop_instance(self.sot)
        self.sot.stop.assert_called_with(
            self.proxy,
        )
        self.proxy._get.assert_called_with(
            _instance.Instance,
            self.sot
        )

    def test_start_instance(self):
        self.sot = _instance.Instance()
        self.sot.start = mock.Mock(return_value={})
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._find = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.start_instance(self.sot)
        self.sot.start.assert_called_with(
            self.proxy,
        )
        self.proxy._get.assert_called_with(
            _instance.Instance,
            self.sot
        )

    def test_restart_instance(self):
        self.sot = _instance.Instance()
        self.sot.restart = mock.Mock(return_value={})
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._find = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.restart_instance(self.sot)
        self.sot.restart.assert_called_with(
            self.proxy,
        )
        self.proxy._get.assert_called_with(
            _instance.Instance,
            self.sot
        )

    def test_change_pwd(self):
        self.sot = _instance.Instance()
        self.sot.change_pwd = mock.Mock(return_value={})
        self.proxy._get = mock.Mock(return_value=self.sot)
        self.proxy._find = mock.Mock(return_value=self.sot)
        self.proxy._get_resource = mock.Mock(return_value=self.sot)

        self.proxy.change_instance_password(self.sot, 'curr', 'new')
        self.sot.change_pwd.assert_called_with(
            self.proxy,
            current_password='curr',
            new_password='new'
        )
        self.proxy._get.assert_not_called()

    def test_statistics(self):
        self.verify_list(
            self.proxy.statistics, _stat.Statistic,
            expected_kwargs={
                'paginated': False
            }
        )

    def test_backups(self):
        self.sot = _backup.Backup()
        self.inst = _instance.Instance(id='1')
        self.proxy._get_resource = mock.Mock(return_value=self.inst)
        self.proxy._list = mock.Mock(return_value=self.sot)

        self.proxy.backups('inst')
        self.proxy._list.assert_called_with(
            _backup.Backup,
            paginated=False,
            instance_id='1'
        )

    def test_backups_query(self):
        self.sot = _backup.Backup()
        self.inst = _instance.Instance(id='1')
        self.proxy._get_resource = mock.Mock(return_value=self.inst)
        self.proxy._list = mock.Mock(return_value=self.sot)

        self.proxy.backups(
            'inst',
            start='1',
            end='2',
            start_time='3',
            end_time='4')
        self.proxy._list.assert_called_with(
            _backup.Backup,
            paginated=False,
            instance_id='1',
            start='1',
            end='2',
            start_time='3',
            end_time='4'
        )

    def test_create_backup(self):
        self.sot = _backup.Backup()
        self.proxy._create = mock.Mock(return_value=self.sot)

        self.proxy.backup_instance('1', remark='rem')
        self.proxy._create.assert_called_with(
            _backup.Backup,
            instance_id='1',
            remark='rem'
        )

    def test_delete_backup(self):
        instance = _instance.Instance(id='instance_id')
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_instance_backup,
            method_args=[instance, 'backup_1'],
            expected_args=[_backup.Backup, 'backup_1'],
            expected_kwargs={
                'instance_id': instance.id,
                'ignore_missing': True
            }
        )

    def test_restores_query(self):
        self.sot = _restore_record.RestoreRecord()
        self.proxy._list = mock.Mock(return_value=self.sot)

        self.proxy.restore_records(
            instance='inst',
            start='1',
            end='2',
            start_time='3',
            end_time='4')
        self.proxy._list.assert_called_with(
            _restore_record.RestoreRecord,
            paginated=False,
            instance_id='inst',
            start='1',
            end='2',
            start_time='3',
            end_time='4'
        )

    def test_restore_backup(self):
        self.proxy._create = mock.Mock()

        self.proxy.restore_instance(
            instance='1',
            backup_id='bck',
            remark='rem')
        self.proxy._create.assert_called_with(
            _restore_record.RestoreRecord,
            instance_id='1',
            backup_id='bck',
            remark='rem'
        )

    def test_configs(self):
        self.proxy._list = mock.Mock()

        self.proxy.instance_params('1')
        self.proxy._list.assert_called_with(
            _config.Config,
            paginated=False,
            instance_id='1'
        )

    def test_update_configs(self):
        self.sot = _config.Config()
        self.proxy._get_resource = mock.Mock(return_value=self.sot)
        self.sot._update = mock.Mock()
        params = []

        self.proxy.update_instance_params(
            instance='inst_id',
            params=params)
        self.sot._update.assert_called_with(
            self.proxy,
            params
        )

    def test_quotas(self):
        self.verify_list(
            self.proxy.quotas, _quota.Quota,
        )

    @mock.patch(
        'otcextensions.sdk.dcs.v1._proxy.Proxy._get_endpoint_with_api_version',
        return_value='fake'
    )
    def test_maintenance_time_windows(self, epo_mock):
        self.verify_list(
            self.proxy.maintenance_time_windows,
            _mtw.MaintenanceTimeWindow
        )
        epo_mock.assert_called_with()

    @mock.patch(
        'otcextensions.sdk.dcs.v1._proxy.Proxy._get_endpoint_with_api_version',
        return_value='fake'
    )
    def test_service_specification(self, epo_mock):
        self.verify_list(
            self.proxy.service_specifications,
            _service_spec.ServiceSpecification
        )
        epo_mock.assert_called_with()

    @mock.patch(
        'otcextensions.sdk.dcs.v1._proxy.Proxy._get_endpoint_with_api_version',
        return_value='fake'
    )
    def test_availability_zones(self, epo_mock):
        self.verify_list(
            self.proxy.availability_zones,
            _az.AvailabilityZone
        )
        epo_mock.assert_called_with()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Message Publishing v2 action implementations"""

import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class PublishMessage(command.ShowOne):
    _description = _("Publish message. Three message formats are supported:\n"
                     "- message\n"
                     "- message_structure\n"
                     "- message_template_name\n"
                     "If the three formats are specified at the same time, "
                     "they take effect in the following sequence: \n"
                     "message_structure > message_template_name > message.")

    def get_parser(self, prog_name):
        parser = super(PublishMessage, self).get_parser(prog_name)
        parser.add_argument(
            'topic',
            metavar='<topic>',
            help=_("Unique resource ID or name of a topic."),
        )
        parser.add_argument(
            '--subject',
            metavar='<subject>',
            required=True,
            help=_("Message subject, which is used as the email "
                   "subject when you publish email messages.."),
        )
        parser.add_argument(
            '--time-to-live',
            metavar='<time_to_live>',
            help=_("Time-to-live (TTL) of a message, specifically "
                   "the maximum time period for retaining the message "
                   "in the system. The default TTL is 3600s (one hour)."),
        )
        parser.add_argument(
            '--message',
            metavar='<message>',
            required=True,
            help=_("Message content. The message content is a UTF-8-coded "
                   "character string of no more than 256 KB."),
        )
        parser.add_argument(
            '--message-structure',
            metavar='<message_structure>',
            help=_("Message structure, which contains JSON character "
                   "strings. Specify protocols in the structure, which "
                   "can be http, https, email, dms, and sms.\n"
                   "The default protocol is mandatory. If the system "
                   "fails to match any other protocols, the default "
                   "message is sent."),
        )
        parser.add_argument(
            '--message-template-name',
            metavar='<message_template_name>',
            help=_("Message template name, which can be obtained "
                   "according to Querying Message Templates."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn

        args_list = [
            'subject',
            'time_to_live',
            'message',
            'message_template_name']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        topic = client.find_topic(parsed_args.topic, ignore_missing=False)
        obj = client.publish_message(topic, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""CSS ELK cluster v1 action implementations"""

import logging
import datetime

from cliff import columns as cliff_columns
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.common import cli_utils
from otcextensions.i18n import _
from otcextensions.osclient.css.v1 import cluster as _cluster

LOG = logging.getLogger(__name__)


FREQUENCY_CHOICES = [
    "day",
    "hour",
    "sun",
    "mon",
    "tue",
    "wed",
    "thu",
    "fri",
    "sat"
]


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class UnixTimestampFormatter(cliff_columns.FormattableColumn):
    """Generate a formatted timestamp."""

    def human_readable(self):
        if self._value is None:
            return ''

        # Convert Unix timestamp to UTC
        if not isinstance(self._value, int):
            self._value = int(self._value) / 1000
        else:
            self._value = self._value / 1000
        return datetime.datetime.fromtimestamp(
            self._value, tz=datetime.timezone.utc
        ).strftime('%Y-%m-%dT%H:%M:%S')


class ListSnapshots(command.Lister):
    _description = _('List CSS Backups')

    columns = (
        'ID',
        'Name',
        'Status',
        'Backup Method',
        'Bucket Name',
        'Created At',
        'Backup Keep Days',
    )

    def get_parser(self, prog_name):
        parser = super(ListSnapshots, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='cluster',
            help=_('Specifies the ID or Name of the CSS Cluster.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(parsed_args.cluster)
        data = client.snapshots(cluster)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class CreateSnapshot(command.ShowOne):
    _description = _('Create a Manual CSS snapshot.')
    columns = ('id', 'name')

    def get_parser(self, prog_name):
        parser = super(CreateSnapshot, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_(
                'Specify ID or Name of the CSS cluster from where '
                'index data is to be backed up.'
            ),
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Snapshot name.'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of a snapshot.'),
        )
        parser.add_argument(
            '--indices',
            metavar='<indices>',
            help=_(
                'Name of the index to be backed up. '
                'Multiple index names are separated by commas.'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster to Restart.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=600,
            help=_('Timeout for the wait in seconds (default: 600 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        attrs = {}

        cluster = client.find_cluster(parsed_args.cluster)
        attrs['name'] = parsed_args.name
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.indices:
            attrs['indices'] = parsed_args.indices

        obj = client.create_snapshot(cluster, **attrs)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)

        data = utils.get_item_properties(obj, self.columns)
        return (self.columns, data)


class ShowSnapshot(command.ShowOne):
    _description = _('Show details of a CSS cluster')

    def get_parser(self, prog_name):
        parser = super(ShowSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('Cluster name or ID.'),
        )
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            help=_('Snapshot name or ID.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        snapshot = client.find_snapshot(
            cluster, parsed_args.snapshot, ignore_missing=False
        )
        _formatters = {
            'start_time': UnixTimestampFormatter,
            'end_time': UnixTimestampFormatter,
            'datastore': cli_utils.YamlFormat,
        }
        display_columns, columns = _get_columns(snapshot)
        data = utils.get_item_properties(
            snapshot, columns, formatters=_formatters
        )

        return (display_columns, data)


class RestoreSnapshot(command.Command):
    _description = _('Restore the CSS cluster using the specified snapshot')

    def get_parser(self, prog_name):
        parser = super(RestoreSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('Cluster ID or Name to which the snapshot belongs.'),
        )
        parser.add_argument(
            'snapshotId',
            metavar='<snapshotId>',
            help=_('The snapshot ID.'),
        )
        parser.add_argument(
            '--target-cluster',
            metavar='<target_cluster>',
            required=True,
            help=_(
                'ID or Name of the cluster, to which the snapshot '
                'is to be restored.'
            ),
        )
        parser.add_argument(
            '--indices',
            metavar='<indices>',
            help=_(
                'Name of the index to be restored. '
                'Multiple index names are separated by commas (,).'
            ),
        )
        parser.add_argument(
            '--rename-pattern',
            metavar='<rename_pattern>',
            help=_(
                'Rule for defining the indices to be restored. '
                'The value contains a maximum of 1,024 characters.'
            ),
        )
        parser.add_argument(
            '--rename-replacement',
            metavar='<rename_replacement>',
            help=_(
                'Rule for renaming an index. '
                'The value contains 0 to 1,024 characters.'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster to Restart.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    @_cluster.translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(parsed_args.cluster)
        target_cluster = client.find_cluster(parsed_args.target_cluster)
        attrs = {'targetCluster': target_cluster.id}
        if parsed_args.rename_replacement:
            attrs['renameReplacement'] = parsed_args.rename_replacement
        if parsed_args.rename_pattern:
            attrs['renamePattern'] = parsed_args.rename_pattern
        if parsed_args.indices:
            attrs['indices'] = parsed_args.indices

        client.restore_snapshot(cluster, parsed_args.snapshotId, **attrs)

        if parsed_args.wait:
            client.wait_for_cluster(target_cluster.id, parsed_args.timeout)

        return client.get_cluster(target_cluster.id)


class SetSnapshotPolicy(command.ShowOne):
    _description = _('Setting the Automatic Snapshot Creation Policy.')

    def get_parser(self, prog_name):
        parser = super(SetSnapshotPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster to which the snapshot belongs.'),
        )
        parser.add_argument(
            "--frequency",
            metavar="{" + ",".join(FREQUENCY_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=FREQUENCY_CHOICES,
            help=_("Frequency of automatically creating snapshots."),
        )
        parser.add_argument(
            '--name-prefix',
            metavar='<name_prefix>',
            required=True,
            help=(
                'Prefix of the snapshot name that is automatically created.'
            ),
        )
        parser.add_argument(
            '--period',
            metavar='<period>',
            default='01:00 GMT+01:00',
            help=(
                'Time when a snapshot is created every day. '
                'Time format is followed by the time zone HH:mm z. '
                'For example, 00:00 GMT+08:00 and 01:00 GMT+08:00. '
                '(Default Value: 01:00 GMT+01:00)'
            ),
        )
        parser.add_argument(
            '--keep-days',
            metavar='<keep_days>',
            required=True,
            type=int,
            help=('Number of days that a snapshot can be retained.'),
        )
        parser.add_argument(
            '--disable',
            action='store_true',
            help=('Disable the automatic snapshot creation policy.'),
        )
        parser.add_argument(
            '--delete-auto',
            action='store_true',
            help=(
                'Whether to delete all automatically created snapshots '
                'when the automatic snapshot creation policy is disabled.'
                'Value is true or false. Default vaule is false'
            ),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        attrs = {
            'prefix': parsed_args.name_prefix,
            'keepday': parsed_args.keep_days,
            'period': parsed_args.period,
            'enable': 'true',
        }
        if getattr(parsed_args, 'disable'):
            attrs['enable'] = 'false'
        if getattr(parsed_args, 'delete_auto'):
            attrs['deleteAuto'] = 'true'
        if parsed_args.frequency:
            attrs['frequency'] = parsed_args.frequency.upper()

        cluster = client.find_cluster(parsed_args.cluster)
        client.set_snapshot_policy(cluster, **attrs)
        return client.get_snapshot_policy(cluster)


class ShowSnapshotPolicy(command.ShowOne):
    _description = _('Show details of a CSS cluster backup policy.')

    def get_parser(self, prog_name):
        parser = super(ShowSnapshotPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster to which the snapshot belongs.'),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(parsed_args.cluster)
        return client.get_snapshot_policy(cluster)


class ConfigureSnapshot(command.Command):
    _description = _('Set Basic Configurations of a CSS Cluster Snapshot.')

    def get_parser(self, prog_name):
        parser = super(ConfigureSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster to which the snapshot belongs.'),
        )
        parser.add_argument(
            '--auto-configure',
            action='store_true',
            help=('Set Snapshot Configuration Automatically.'),
        )
        parser.add_argument(
            '--backup-path',
            metavar='<backup_path>',
            help=('Storage path of the snapshot in the OBS bucket.'),
        )
        parser.add_argument(
            '--bucket',
            metavar='<bucket>',
            help=('OBS bucket used for index data backup.'),
        )
        parser.add_argument(
            '--agency',
            metavar='<agency>',
            help=('IAM agency used to access OBS.'),
        )
        parser.add_argument(
            '--cmk-id',
            metavar='<cmk_id>',
            help=('Key ID used for snapshot encryption.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        attrs = {}
        if not parsed_args.auto_configure:
            mandatory_args = {
                'bucket': 'Missing mandatory argument: --bucket',
                'agency': 'Missing mandatory argument: --agency',
                'backup_path': 'Missing mandatory argument: --backup-path',
            }

            for arg, error_msg in mandatory_args.items():
                value = getattr(parsed_args, arg, None)
                if not value:
                    raise exceptions.CommandError(error_msg)
                attrs[arg] = value

            if parsed_args.cmk_id:
                attrs['snapshotCmkId'] = parsed_args.cmk_id

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        client.set_snapshot_configuration(
            cluster, auto_configure=parsed_args.auto_configure, **attrs
        )


class DeleteSnapshot(command.Command):
    _description = _('Delete CSS Cluster Snapshot(s).')

    def get_parser(self, prog_name):
        parser = super(DeleteSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster to which the snapshot belongs.'),
        )
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            nargs='+',
            help=_('ID(s) of the snapshot(s) to be deleted.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(parsed_args.cluster)
        result = 0
        for snapshotId in parsed_args.snapshot:
            try:
                client.delete_snapshot(
                    cluster, snapshotId, ignore_missing=False
                )
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        'Failed to delete Snapshot(s) with '
                        "ID or Name '%(snapshot)s': %(e)s"
                    ),
                    {'snapshot': snapshotId, 'e': e},
                )
        if result > 0:
            total = len(parsed_args.snapshot)
            msg = _(
                '%(result)s of %(total)s Snapshot(s) failed ' 'to delete.'
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)


class DisableSnapshot(command.Command):
    _description = _('Disable the snapshot creation function.')

    def get_parser(self, prog_name):
        parser = super(DisableSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the cluster to which the snapshot belongs.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(parsed_args.cluster)
        client.disable_snapshot_function(cluster)
